@@ -13,7 +13,6 @@ mod container;
 mod create_exe;
 #[cfg(feature = "static-artifact-create")]
 mod create_obj;
-pub(crate) mod deploy;
 pub(crate) mod domain;
 #[cfg(feature = "static-artifact-create")]
 mod gen_c_header;
@@ -24,7 +23,6 @@ mod journal;
 mod login;
 pub(crate) mod namespace;
 mod package;
-mod publish;
 mod run;
 mod self_update;
 pub mod ssh;
@@ -131,10 +129,13 @@ impl WasmerCmd {
             Some(Cmd::Inspect(inspect)) => inspect.execute(),
             Some(Cmd::Init(init)) => init.execute(),
             Some(Cmd::Login(login)) => login.execute(),
-            Some(Cmd::Publish(publish)) => publish.execute(),
+            Some(Cmd::Publish(publish)) => publish.run().map(|_| ()),
             Some(Cmd::Package(cmd)) => match cmd {
                 Package::Download(cmd) => cmd.execute(),
-                Package::Build(cmd) => cmd.execute(),
+                Package::Build(cmd) => cmd.execute().map(|_| ()),
+                Package::Tag(cmd) => cmd.run(),
+                Package::Push(cmd) => cmd.run(),
+                Package::Publish(cmd) => cmd.run().map(|_| ()),
             },
             Some(Cmd::Container(cmd)) => match cmd {
                 crate::commands::Container::Unpack(cmd) => cmd.execute(),
@@ -215,9 +216,9 @@ enum Cmd {
     /// Login into a wasmer.io-like registry
     Login(Login),
 
-    /// Login into a wasmer.io-like registry
+    /// Publish a package to a registry [alias: package publish]
     #[clap(name = "publish")]
-    Publish(Publish),
+    Publish(crate::commands::package::publish::PackagePublish),
 
     /// Wasmer cache
     Cache(Cache),
@@ -326,10 +327,10 @@ enum Cmd {
     /// Shows the current logged in user for the current active registry
     Whoami(Whoami),
 
-    /// Add a Wasmer package's bindings to your application.
+    /// Add a Wasmer package's bindings to your application
     Add(Add),
 
-    /// Run a WebAssembly file or Wasmer container.
+    /// Run a WebAssembly file or Wasmer container
     #[clap(alias = "run-unstable")]
     Run(Run),
 
@@ -345,17 +346,17 @@ enum Cmd {
     Container(crate::commands::Container),
 
     // Edge commands
-    /// Deploy apps to Wasmer Edge.
-    Deploy(crate::commands::deploy::CmdDeploy),
+    /// Deploy apps to Wasmer Edge [alias: app deploy]
+    Deploy(crate::commands::app::deploy::CmdAppDeploy),
 
-    /// Manage deployed Edge apps.
+    /// Manage deployed Edge apps
     #[clap(subcommand, alias = "apps")]
     App(crate::commands::app::CmdApp),
 
-    /// Run commands/packages on Wasmer Edge in an interactive shell session.
+    /// Run commands/packages on Wasmer Edge in an interactive shell session
     Ssh(crate::commands::ssh::CmdSsh),
 
-    /// Manage Wasmer namespaces.
+    /// Manage Wasmer namespaces
     #[clap(subcommand, alias = "namespaces")]
     Namespace(crate::commands::namespace::CmdNamespace),
 
