@@ -1,5 +1,5 @@
 // This file contains code from external sources.
-// Attributions: https://github.com/wasmerio/wasmer/blob/master/ATTRIBUTIONS.md
+// Attributions: https://github.com/wasmerio/wasmer/blob/main/ATTRIBUTIONS.md
 
 //! Boxed slices for `PrimaryMap`.
 
@@ -10,8 +10,6 @@ use crate::lib::std::boxed::Box;
 use crate::lib::std::marker::PhantomData;
 use crate::lib::std::ops::{Index, IndexMut};
 use crate::lib::std::slice;
-use loupe::{MemoryUsage, MemoryUsageTracker};
-use std::mem;
 
 /// A slice mapping `K -> V` allocating dense entity references.
 ///
@@ -146,21 +144,6 @@ where
     }
 }
 
-impl<K, V> MemoryUsage for BoxedSlice<K, V>
-where
-    K: EntityRef,
-    V: MemoryUsage,
-{
-    fn size_of_val(&self, tracker: &mut dyn MemoryUsageTracker) -> usize {
-        mem::size_of_val(self)
-            + self
-                .elems
-                .iter()
-                .map(|value| value.size_of_val(tracker) - mem::size_of_val(value))
-                .sum::<usize>()
-    }
-}
-
 #[cfg(test)]
 mod tests {
     use super::*;
