@@ -118,6 +118,10 @@ where
         let inner = self.inner.lock().unwrap();
         inner.created_time()
     }
+    fn set_times(&mut self, atime: Option<u64>, mtime: Option<u64>) -> crate::Result<()> {
+        let mut inner = self.inner.lock().unwrap();
+        inner.set_times(atime, mtime)
+    }
     fn size(&self) -> u64 {
         let inner = self.inner.lock().unwrap();
         inner.size()
