@@ -1,5 +1,7 @@
 //! This module contains the [`FileSystem`] type itself.
 
+use self::offloaded_file::OffloadBackingStore;
+
 use super::*;
 use crate::{DirEntry, FileSystem as _, FileType, FsError, Metadata, OpenOptions, ReadDir, Result};
 use futures::future::BoxFuture;
@@ -29,6 +31,20 @@ impl FileSystem {
         self
     }
 
+    /// Uses a mmap'ed file as a cache for file data thus removing the
+    /// need to copy the data into memory.
+    ///
+    /// This is especially important for journals as it means that the
+    /// data stored within the journals does not need to be copied
+    /// into memory, for very large journals this would otherwise be
+    /// a problem.
+    pub fn with_backing_offload(self, buffer: OffloadBackingStore) -> Result<Self> {
+        let mut lock = self.inner.write().map_err(|_| FsError::Lock)?;
+        lock.backing_offload.replace(buffer);
+        drop(lock);
+        Ok(self)
+    }
+
     /// Canonicalize a path without validating that it actually exists.
     pub fn canonicalize_unchecked(&self, path: &Path) -> Result<PathBuf> {
         let lock = self.inner.read().map_err(|_| FsError::Lock)?;
@@ -619,6 +635,7 @@ impl fmt::Debug for FileSystem {
 /// indexed by their respective `Inode` in a slab.
 pub(super) struct FileSystemInner {
     pub(super) storage: Slab<Node>,
+    pub(super) backing_offload: Option<OffloadBackingStore>,
     pub(super) limiter: Option<crate::limiter::DynFsMemoryLimiter>,
 }
 
@@ -754,6 +771,7 @@ impl FileSystemInner {
                 .filter_map(|(nth, inode)| self.storage.get(*inode).map(|node| (nth, node)))
                 .find_map(|(nth, node)| match node {
                     Node::File(FileNode { inode, name, .. })
+                    | Node::OffloadedFile(OffloadedFileNode { inode, name, .. })
                     | Node::ReadOnlyFile(ReadOnlyFileNode { inode, name, .. })
                     | Node::CustomFile(CustomFileNode { inode, name, .. })
                     | Node::ArcFile(ArcFileNode { inode, name, .. })
@@ -793,6 +811,7 @@ impl FileSystemInner {
                 .filter_map(|(nth, inode)| self.storage.get(*inode).map(|node| (nth, node)))
                 .find_map(|(nth, node)| match node {
                     Node::File(FileNode { inode, name, .. })
+                    | Node::OffloadedFile(OffloadedFileNode { inode, name, .. })
                     | Node::Directory(DirectoryNode { inode, name, .. })
                     | Node::ReadOnlyFile(ReadOnlyFileNode { inode, name, .. })
                     | Node::CustomFile(CustomFileNode { inode, name, .. })
@@ -956,6 +975,7 @@ impl fmt::Debug for FileSystemInner {
                     inode = node.inode(),
                     ty = match node {
                         Node::File { .. } => "file",
+                        Node::OffloadedFile { .. } => "offloaded-file",
                         Node::ReadOnlyFile { .. } => "ro-file",
                         Node::ArcFile { .. } => "arc-file",
                         Node::CustomFile { .. } => "custom-file",
@@ -1015,6 +1035,7 @@ impl Default for FileSystemInner {
 
         Self {
             storage: slab,
+            backing_offload: None,
             limiter: None,
         }
     }
