@@ -2,19 +2,24 @@
 
 #[cfg(feature = "remote")]
 pub mod client;
+pub mod composite;
 #[cfg(feature = "host-net")]
 pub mod host;
+pub mod loopback;
 pub mod meta;
 #[cfg(feature = "remote")]
 pub mod rx_tx;
 #[cfg(feature = "remote")]
 pub mod server;
+pub mod tcp_pair;
 #[cfg(feature = "tokio")]
 #[cfg(test)]
 mod tests;
 
 #[cfg(feature = "remote")]
 pub use client::{RemoteNetworkingClient, RemoteNetworkingClientDriver};
+pub use composite::CompositeTcpListener;
+pub use loopback::LoopbackNetworking;
 use pin_project_lite::pin_project;
 #[cfg(feature = "rkyv")]
 use rkyv::{Archive, CheckBytes, Deserialize as RkyvDeserialize, Serialize as RkyvSerialize};
