@@ -1,19 +1,18 @@
-use std::sync::Arc;
+use std::{path::Path, sync::Arc};
 
 use anyhow::Context;
 use derivative::*;
 use once_cell::sync::OnceCell;
-use semver::Version;
+use sha2::Digest;
 use virtual_fs::FileSystem;
+use wasmer_config::package::{PackageHash, PackageId, PackageSource};
 use webc::{compat::SharedBytes, Container};
 
 use crate::{
-    runtime::{
-        module_cache::ModuleHash,
-        resolver::{PackageId, PackageInfo, PackageSpecifier, ResolveError},
-    },
+    runtime::resolver::{PackageInfo, ResolveError},
     Runtime,
 };
+use wasmer_types::ModuleHash;
 
 #[derive(Derivative, Clone)]
 #[derivative(Debug)]
@@ -22,16 +21,21 @@ pub struct BinaryPackageCommand {
     metadata: webc::metadata::Command,
     #[derivative(Debug = "ignore")]
     pub(crate) atom: SharedBytes,
-    hash: OnceCell<ModuleHash>,
+    hash: ModuleHash,
 }
 
 impl BinaryPackageCommand {
-    pub fn new(name: String, metadata: webc::metadata::Command, atom: SharedBytes) -> Self {
+    pub fn new(
+        name: String,
+        metadata: webc::metadata::Command,
+        atom: SharedBytes,
+        hash: ModuleHash,
+    ) -> Self {
         Self {
             name,
             metadata,
             atom,
-            hash: OnceCell::new(),
+            hash,
         }
     }
 
@@ -52,7 +56,7 @@ impl BinaryPackageCommand {
     }
 
     pub fn hash(&self) -> &ModuleHash {
-        self.hash.get_or_init(|| ModuleHash::hash(self.atom()))
+        &self.hash
     }
 }
 
@@ -60,7 +64,8 @@ impl BinaryPackageCommand {
 #[derive(Derivative, Clone)]
 #[derivative(Debug)]
 pub struct BinaryPackage {
-    pub package_name: String,
+    pub id: PackageId,
+
     pub when_cached: Option<u128>,
     /// The name of the [`BinaryPackageCommand`] which is this package's
     /// entrypoint.
@@ -69,11 +74,40 @@ pub struct BinaryPackage {
     pub webc_fs: Arc<dyn FileSystem + Send + Sync>,
     pub commands: Vec<BinaryPackageCommand>,
     pub uses: Vec<String>,
-    pub version: Version,
     pub file_system_memory_footprint: u64,
 }
 
 impl BinaryPackage {
+    #[tracing::instrument(level = "debug", skip_all)]
+    pub async fn from_dir(
+        dir: &Path,
+        rt: &(dyn Runtime + Send + Sync),
+    ) -> Result<Self, anyhow::Error> {
+        let source = rt.source();
+
+        // since each package must be in its own directory, hash of the `dir` should provide a good enough
+        // unique identifier for the package
+        let hash = sha2::Sha256::digest(dir.display().to_string().as_bytes()).into();
+        let id = PackageId::Hash(PackageHash::from_sha256_bytes(hash));
+
+        let manifest_path = dir.join("wasmer.toml");
+        let webc = webc::wasmer_package::Package::from_manifest(manifest_path)?;
+        let container = Container::from(webc);
+        let manifest = container.manifest();
+
+        let root = PackageInfo::from_manifest(id, manifest, container.version())?;
+        let root_id = root.id.clone();
+
+        let resolution = crate::runtime::resolver::resolve(&root_id, &root, &*source).await?;
+        let pkg = rt
+            .package_loader()
+            .load_package_tree(&container, &resolution)
+            .await
+            .map_err(|e| anyhow::anyhow!(e))?;
+
+        Ok(pkg)
+    }
+
     /// Load a [`webc::Container`] and all its dependencies into a
     /// [`BinaryPackage`].
     #[tracing::instrument(level = "debug", skip_all)]
@@ -82,11 +116,18 @@ impl BinaryPackage {
         rt: &(dyn Runtime + Send + Sync),
     ) -> Result<Self, anyhow::Error> {
         let source = rt.source();
-        let root = PackageInfo::from_manifest(container.manifest())?;
-        let root_id = PackageId {
-            package_name: root.name.clone(),
-            version: root.version.clone(),
-        };
+
+        let manifest = container.manifest();
+        let id = PackageInfo::package_id_from_manifest(manifest)?
+            .or_else(|| {
+                container
+                    .webc_hash()
+                    .map(|hash| PackageId::Hash(PackageHash::from_sha256_bytes(hash)))
+            })
+            .ok_or_else(|| anyhow::Error::msg("webc file did not provide its hash"))?;
+
+        let root = PackageInfo::from_manifest(id, manifest, container.version())?;
+        let root_id = root.id.clone();
 
         let resolution = crate::runtime::resolver::resolve(&root_id, &root, &*source).await?;
         let pkg = rt
@@ -101,7 +142,7 @@ impl BinaryPackage {
     /// Load a [`BinaryPackage`] and all its dependencies from a registry.
     #[tracing::instrument(level = "debug", skip_all)]
     pub async fn from_registry(
-        specifier: &PackageSpecifier,
+        specifier: &PackageSource,
         runtime: &(dyn Runtime + Send + Sync),
     ) -> Result<Self, anyhow::Error> {
         let source = runtime.source();
@@ -143,9 +184,9 @@ impl BinaryPackage {
     pub fn hash(&self) -> ModuleHash {
         *self.hash.get_or_init(|| {
             if let Some(entry) = self.entrypoint_bytes() {
-                ModuleHash::hash(entry)
+                ModuleHash::xxhash(entry)
             } else {
-                ModuleHash::hash(self.package_name.as_bytes())
+                ModuleHash::xxhash(self.id.to_string())
             }
         })
     }
@@ -153,6 +194,7 @@ impl BinaryPackage {
 
 #[cfg(test)]
 mod tests {
+    use sha2::Digest;
     use tempfile::TempDir;
     use virtual_fs::AsyncReadExt;
 
@@ -195,9 +237,6 @@ mod tests {
         std::fs::create_dir_all(&out).unwrap();
         let file_txt = "Hello, World!";
         std::fs::write(out.join("file.txt"), file_txt).unwrap();
-        let webc: Container = webc::wasmer_package::Package::from_manifest(manifest)
-            .unwrap()
-            .into();
         let tasks = task_manager();
         let mut runtime = PluggableRuntime::new(tasks);
         runtime.set_package_loader(
@@ -205,7 +244,9 @@ mod tests {
                 .with_shared_http_client(runtime.http_client().unwrap().clone()),
         );
 
-        let pkg = BinaryPackage::from_webc(&webc, &runtime).await.unwrap();
+        let pkg = BinaryPackage::from_dir(&temp.path(), &runtime)
+            .await
+            .unwrap();
 
         // We should have mapped "./out/file.txt" on the host to
         // "/public/file.txt" on the guest.
@@ -219,4 +260,54 @@ mod tests {
         f.read_to_string(&mut buffer).await.unwrap();
         assert_eq!(buffer, file_txt);
     }
+
+    #[tokio::test]
+    #[cfg_attr(
+        not(feature = "sys-thread"),
+        ignore = "The tokio task manager isn't available on this platform"
+    )]
+    async fn commands_use_the_atom_signature() {
+        let temp = TempDir::new().unwrap();
+        let wasmer_toml = r#"
+            [package]
+            name = "some/package"
+            version = "0.0.0"
+            description = "a dummy package"
+
+            [[module]]
+            name = "foo"
+            source = "foo.wasm"
+            abi = "wasi"
+            
+            [[command]]
+            name = "cmd"
+            module = "foo"     
+        "#;
+        let manifest = temp.path().join("wasmer.toml");
+        std::fs::write(&manifest, wasmer_toml).unwrap();
+
+        let atom_path = temp.path().join("foo.wasm");
+        std::fs::write(&atom_path, b"").unwrap();
+
+        let webc: Container = webc::wasmer_package::Package::from_manifest(&manifest)
+            .unwrap()
+            .into();
+
+        let tasks = task_manager();
+        let mut runtime = PluggableRuntime::new(tasks);
+        runtime.set_package_loader(
+            BuiltinPackageLoader::new()
+                .with_shared_http_client(runtime.http_client().unwrap().clone()),
+        );
+
+        let pkg = BinaryPackage::from_dir(&temp.path(), &runtime)
+            .await
+            .unwrap();
+
+        assert_eq!(pkg.commands.len(), 1);
+        let command = pkg.get_command("cmd").unwrap();
+        let atom_sha256_hash: [u8; 32] = sha2::Sha256::digest(webc.get_atom("foo").unwrap()).into();
+        let module_hash = ModuleHash::sha256_from_bytes(atom_sha256_hash);
+        assert_eq!(command.hash(), &module_hash);
+    }
 }
