@@ -1,3 +1,5 @@
+use crate::VIRTUAL_ROOT_FD;
+
 use super::*;
 
 impl JournalEffector {
@@ -32,19 +34,24 @@ impl JournalEffector {
         st_mtim: Timestamp,
         fst_flags: Fstflags,
     ) -> anyhow::Result<()> {
-        crate::syscalls::path_filestat_set_times_internal(ctx, fd, flags, path, st_atim, st_mtim, fst_flags)
-            .map_err(|err| {
-                anyhow::format_err!(
-                    "journal restore error: failed to set path times (fd={}, flags={}, path={}, st_atim={}, st_mtim={}, fst_flags={:?}) - {}",
-                    fd,
-                    flags,
-                    path,
-                    st_atim,
-                    st_mtim,
-                    fst_flags,
-                    err
-                )
-            })?;
+        // see `VIRTUAL_ROOT_FD` for details as to why this exists
+        if fd == VIRTUAL_ROOT_FD {
+            // we ignore this record as its not implemented yet
+        } else {
+            crate::syscalls::path_filestat_set_times_internal(ctx, fd, flags, path, st_atim, st_mtim, fst_flags)
+                .map_err(|err| {
+                    anyhow::format_err!(
+                        "journal restore error: failed to set path times (fd={}, flags={}, path={}, st_atim={}, st_mtim={}, fst_flags={:?}) - {}",
+                        fd,
+                        flags,
+                        path,
+                        st_atim,
+                        st_mtim,
+                        fst_flags,
+                        err
+                    )
+                })?;
+        }
         Ok(())
     }
 }
