@@ -35,27 +35,32 @@ pub enum InodeHttpSocketType {
 
 #[derive(Derivative)]
 #[derivative(Debug)]
+pub struct SocketProperties {
+    pub family: Addressfamily,
+    pub ty: Socktype,
+    pub pt: SockProto,
+    pub only_v6: bool,
+    pub reuse_port: bool,
+    pub reuse_addr: bool,
+    pub no_delay: Option<bool>,
+    pub keep_alive: Option<bool>,
+    pub dont_route: Option<bool>,
+    pub send_buf_size: Option<usize>,
+    pub recv_buf_size: Option<usize>,
+    pub write_timeout: Option<Duration>,
+    pub read_timeout: Option<Duration>,
+    pub accept_timeout: Option<Duration>,
+    pub connect_timeout: Option<Duration>,
+    #[derivative(Debug = "ignore")]
+    pub handler: Option<Box<dyn InterestHandler + Send + Sync>>,
+}
+
+#[derive(Debug)]
 //#[cfg_attr(feature = "enable-serde", derive(Serialize, Deserialize))]
 pub enum InodeSocketKind {
     PreSocket {
-        family: Addressfamily,
-        ty: Socktype,
-        pt: SockProto,
+        props: SocketProperties,
         addr: Option<SocketAddr>,
-        only_v6: bool,
-        reuse_port: bool,
-        reuse_addr: bool,
-        no_delay: Option<bool>,
-        keep_alive: Option<bool>,
-        dont_route: Option<bool>,
-        send_buf_size: Option<usize>,
-        recv_buf_size: Option<usize>,
-        write_timeout: Option<Duration>,
-        read_timeout: Option<Duration>,
-        accept_timeout: Option<Duration>,
-        connect_timeout: Option<Duration>,
-        #[derivative(Debug = "ignore")]
-        handler: Option<Box<dyn InterestHandler + Send + Sync>>,
     },
     Icmp(Box<dyn VirtualIcmpSocket + Sync>),
     Raw(Box<dyn VirtualRawSocket + Sync>),
@@ -72,8 +77,12 @@ pub enum InodeSocketKind {
         socket: Box<dyn VirtualUdpSocket + Sync>,
         peer: Option<SocketAddr>,
     },
-    RemoteTcpStream {
+    RemoteSocket {
+        props: SocketProperties,
+        local_addr: SocketAddr,
         peer_addr: SocketAddr,
+        ttl: u32,
+        multicast_ttl: u32,
     },
 }
 
@@ -241,15 +250,8 @@ impl InodeSocket {
         let socket = {
             let mut inner = self.inner.protected.write().unwrap();
             match &mut inner.kind {
-                InodeSocketKind::PreSocket {
-                    family,
-                    ty,
-                    addr,
-                    reuse_port,
-                    reuse_addr,
-                    ..
-                } => {
-                    match *family {
+                InodeSocketKind::PreSocket { props, addr, .. } => {
+                    match props.family {
                         Addressfamily::Inet4 => {
                             if !set_addr.is_ipv4() {
                                 tracing::debug!(
@@ -274,15 +276,61 @@ impl InodeSocket {
                     addr.replace(set_addr);
                     let addr = (*addr).unwrap();
 
-                    match *ty {
+                    match props.ty {
                         Socktype::Stream => {
                             // we already set the socket address - next we need a listen or connect so nothing
                             // more to do at this time
                             return Ok(None);
                         }
                         Socktype::Dgram => {
-                            let reuse_port = *reuse_port;
-                            let reuse_addr = *reuse_addr;
+                            let reuse_port = props.reuse_port;
+                            let reuse_addr = props.reuse_addr;
+                            drop(inner);
+
+                            net.bind_udp(addr, reuse_port, reuse_addr)
+                        }
+                        _ => return Err(Errno::Inval),
+                    }
+                }
+                InodeSocketKind::RemoteSocket {
+                    props,
+                    local_addr: addr,
+                    ..
+                } => {
+                    match props.family {
+                        Addressfamily::Inet4 => {
+                            if !set_addr.is_ipv4() {
+                                tracing::debug!(
+                                    "IP address is the wrong type IPv4 ({set_addr}) vs IPv6 family"
+                                );
+                                return Err(Errno::Inval);
+                            }
+                        }
+                        Addressfamily::Inet6 => {
+                            if !set_addr.is_ipv6() {
+                                tracing::debug!(
+                                    "IP address is the wrong type IPv6 ({set_addr}) vs IPv4 family"
+                                );
+                                return Err(Errno::Inval);
+                            }
+                        }
+                        _ => {
+                            return Err(Errno::Notsup);
+                        }
+                    }
+
+                    *addr = set_addr;
+                    let addr = *addr;
+
+                    match props.ty {
+                        Socktype::Stream => {
+                            // we already set the socket address - next we need a listen or connect so nothing
+                            // more to do at this time
+                            return Ok(None);
+                        }
+                        Socktype::Dgram => {
+                            let reuse_port = props.reuse_port;
+                            let reuse_addr = props.reuse_addr;
                             drop(inner);
 
                             net.bind_udp(addr, reuse_port, reuse_addr)
@@ -318,34 +366,70 @@ impl InodeSocket {
         let socket = {
             let inner = self.inner.protected.read().unwrap();
             match &inner.kind {
-                InodeSocketKind::PreSocket {
-                    ty,
-                    addr,
-                    only_v6,
-                    reuse_port,
-                    reuse_addr,
-                    ..
-                } => match *ty {
+                InodeSocketKind::PreSocket { props, addr, .. } => match props.ty {
                     Socktype::Stream => {
                         if addr.is_none() {
                             tracing::warn!("wasi[?]::sock_listen - failed - address not set");
                             return Err(Errno::Inval);
                         }
                         let addr = *addr.as_ref().unwrap();
-                        let only_v6 = *only_v6;
-                        let reuse_port = *reuse_port;
-                        let reuse_addr = *reuse_addr;
+                        let only_v6 = props.only_v6;
+                        let reuse_port = props.reuse_port;
+                        let reuse_addr = props.reuse_addr;
                         drop(inner);
 
                         net.listen_tcp(addr, only_v6, reuse_port, reuse_addr)
                     }
-                    _ => {
-                        tracing::warn!("wasi[?]::sock_listen - failed - not supported(1)");
+                    ty => {
+                        tracing::warn!(
+                            "wasi[?]::sock_listen - failed - not supported(pre-socket:{:?})",
+                            ty
+                        );
                         return Err(Errno::Notsup);
                     }
                 },
-                _ => {
-                    tracing::warn!("wasi[?]::sock_listen - failed - not supported(2)");
+                InodeSocketKind::RemoteSocket {
+                    props,
+                    local_addr: addr,
+                    ..
+                } => match props.ty {
+                    Socktype::Stream => {
+                        let addr = *addr;
+                        let only_v6 = props.only_v6;
+                        let reuse_port = props.reuse_port;
+                        let reuse_addr = props.reuse_addr;
+                        drop(inner);
+
+                        net.listen_tcp(addr, only_v6, reuse_port, reuse_addr)
+                    }
+                    ty => {
+                        tracing::warn!(
+                            "wasi[?]::sock_listen - failed - not supported(remote-socket:{:?})",
+                            ty
+                        );
+                        return Err(Errno::Notsup);
+                    }
+                },
+                InodeSocketKind::Icmp(_) => {
+                    tracing::warn!("wasi[?]::sock_listen - failed - not supported(icmp)");
+                    return Err(Errno::Notsup);
+                }
+                InodeSocketKind::Raw(_) => {
+                    tracing::warn!("wasi[?]::sock_listen - failed - not supported(raw)");
+                    return Err(Errno::Notsup);
+                }
+                InodeSocketKind::TcpListener { .. } => {
+                    tracing::warn!(
+                        "wasi[?]::sock_listen - failed - already listening (tcp-listener)"
+                    );
+                    return Err(Errno::Notsup);
+                }
+                InodeSocketKind::TcpStream { .. } => {
+                    tracing::warn!("wasi[?]::sock_listen - failed - not supported(tcp-stream)");
+                    return Err(Errno::Notsup);
+                }
+                InodeSocketKind::UdpSocket { .. } => {
+                    tracing::warn!("wasi[?]::sock_listen - failed - not supported(udp-socket)");
                     return Err(Errno::Notsup);
                 }
             }
@@ -441,7 +525,7 @@ impl InodeSocket {
             InodeSocketKind::UdpSocket { .. } => {}
             InodeSocketKind::Raw(_) => {}
             InodeSocketKind::PreSocket { .. } => return Err(Errno::Notconn),
-            InodeSocketKind::RemoteTcpStream { .. } => {}
+            InodeSocketKind::RemoteSocket { .. } => {}
         };
         Ok(())
     }
@@ -462,25 +546,15 @@ impl InodeSocket {
         let connect = {
             let mut inner = self.inner.protected.write().unwrap();
             match &mut inner.kind {
-                InodeSocketKind::PreSocket {
-                    ty,
-                    addr,
-                    write_timeout,
-                    read_timeout,
-                    no_delay,
-                    keep_alive,
-                    dont_route,
-                    handler: h,
-                    ..
-                } => {
-                    handler = h.take();
-                    new_write_timeout = *write_timeout;
-                    new_read_timeout = *read_timeout;
-                    match *ty {
+                InodeSocketKind::PreSocket { props, addr, .. } => {
+                    handler = props.handler.take();
+                    new_write_timeout = props.write_timeout;
+                    new_read_timeout = props.read_timeout;
+                    match props.ty {
                         Socktype::Stream => {
-                            let no_delay = *no_delay;
-                            let keep_alive = *keep_alive;
-                            let dont_route = *dont_route;
+                            let no_delay = props.no_delay;
+                            let keep_alive = props.keep_alive;
+                            let dont_route = props.dont_route;
                             let addr = match addr {
                                 Some(a) => *a,
                                 None => {
@@ -515,6 +589,10 @@ impl InodeSocket {
                     target_peer.replace(peer);
                     return Ok(None);
                 }
+                InodeSocketKind::RemoteSocket { peer_addr, .. } => {
+                    *peer_addr = peer;
+                    return Ok(None);
+                }
                 _ => return Err(Errno::Notsup),
             }
         };
@@ -546,6 +624,7 @@ impl InodeSocket {
             InodeSocketKind::TcpListener { .. } => WasiSocketStatus::Opened,
             InodeSocketKind::TcpStream { .. } => WasiSocketStatus::Opened,
             InodeSocketKind::UdpSocket { .. } => WasiSocketStatus::Opened,
+            InodeSocketKind::RemoteSocket { .. } => WasiSocketStatus::Opened,
             _ => WasiSocketStatus::Failed,
         })
     }
@@ -553,12 +632,12 @@ impl InodeSocket {
     pub fn addr_local(&self) -> Result<SocketAddr, Errno> {
         let inner = self.inner.protected.read().unwrap();
         Ok(match &inner.kind {
-            InodeSocketKind::PreSocket { family, addr, .. } => {
+            InodeSocketKind::PreSocket { props, addr, .. } => {
                 if let Some(addr) = addr {
                     *addr
                 } else {
                     SocketAddr::new(
-                        match *family {
+                        match props.family {
                             Addressfamily::Inet4 => IpAddr::V4(Ipv4Addr::UNSPECIFIED),
                             Addressfamily::Inet6 => IpAddr::V6(Ipv6Addr::UNSPECIFIED),
                             _ => return Err(Errno::Inval),
@@ -577,6 +656,9 @@ impl InodeSocket {
             InodeSocketKind::UdpSocket { socket, .. } => {
                 socket.addr_local().map_err(net_error_into_wasi_err)?
             }
+            InodeSocketKind::RemoteSocket {
+                local_addr: addr, ..
+            } => *addr,
             _ => return Err(Errno::Notsup),
         })
     }
@@ -584,8 +666,8 @@ impl InodeSocket {
     pub fn addr_peer(&self) -> Result<SocketAddr, Errno> {
         let inner = self.inner.protected.read().unwrap();
         Ok(match &inner.kind {
-            InodeSocketKind::PreSocket { family, .. } => SocketAddr::new(
-                match *family {
+            InodeSocketKind::PreSocket { props, .. } => SocketAddr::new(
+                match props.family {
                     Addressfamily::Inet4 => IpAddr::V4(Ipv4Addr::UNSPECIFIED),
                     Addressfamily::Inet6 => IpAddr::V6(Ipv6Addr::UNSPECIFIED),
                     _ => return Err(Errno::Inval),
@@ -613,6 +695,7 @@ impl InodeSocket {
                             )
                         })
                 })?,
+            InodeSocketKind::RemoteSocket { peer_addr, .. } => *peer_addr,
             _ => return Err(Errno::Notsup),
         })
     }
@@ -620,22 +703,15 @@ impl InodeSocket {
     pub fn set_opt_flag(&mut self, option: WasiSocketOption, val: bool) -> Result<(), Errno> {
         let mut inner = self.inner.protected.write().unwrap();
         match &mut inner.kind {
-            InodeSocketKind::PreSocket {
-                only_v6,
-                reuse_port,
-                reuse_addr,
-                no_delay,
-                keep_alive,
-                dont_route,
-                ..
-            } => {
+            InodeSocketKind::PreSocket { props, .. }
+            | InodeSocketKind::RemoteSocket { props, .. } => {
                 match option {
-                    WasiSocketOption::OnlyV6 => *only_v6 = val,
-                    WasiSocketOption::ReusePort => *reuse_port = val,
-                    WasiSocketOption::ReuseAddr => *reuse_addr = val,
-                    WasiSocketOption::NoDelay => *no_delay = Some(val),
-                    WasiSocketOption::KeepAlive => *keep_alive = Some(val),
-                    WasiSocketOption::DontRoute => *dont_route = Some(val),
+                    WasiSocketOption::OnlyV6 => props.only_v6 = val,
+                    WasiSocketOption::ReusePort => props.reuse_port = val,
+                    WasiSocketOption::ReuseAddr => props.reuse_addr = val,
+                    WasiSocketOption::NoDelay => props.no_delay = Some(val),
+                    WasiSocketOption::KeepAlive => props.keep_alive = Some(val),
+                    WasiSocketOption::DontRoute => props.dont_route = Some(val),
                     _ => return Err(Errno::Inval),
                 };
             }
@@ -678,19 +754,13 @@ impl InodeSocket {
     pub fn get_opt_flag(&self, option: WasiSocketOption) -> Result<bool, Errno> {
         let mut inner = self.inner.protected.write().unwrap();
         Ok(match &mut inner.kind {
-            InodeSocketKind::PreSocket {
-                only_v6,
-                reuse_port,
-                reuse_addr,
-                no_delay,
-                keep_alive,
-                ..
-            } => match option {
-                WasiSocketOption::OnlyV6 => *only_v6,
-                WasiSocketOption::ReusePort => *reuse_port,
-                WasiSocketOption::ReuseAddr => *reuse_addr,
-                WasiSocketOption::NoDelay => no_delay.unwrap_or_default(),
-                WasiSocketOption::KeepAlive => keep_alive.unwrap_or_default(),
+            InodeSocketKind::PreSocket { props, .. }
+            | InodeSocketKind::RemoteSocket { props, .. } => match option {
+                WasiSocketOption::OnlyV6 => props.only_v6,
+                WasiSocketOption::ReusePort => props.reuse_port,
+                WasiSocketOption::ReuseAddr => props.reuse_addr,
+                WasiSocketOption::NoDelay => props.no_delay.unwrap_or_default(),
+                WasiSocketOption::KeepAlive => props.keep_alive.unwrap_or_default(),
                 _ => return Err(Errno::Inval),
             },
             InodeSocketKind::Raw(sock) => match option {
@@ -728,8 +798,9 @@ impl InodeSocket {
     pub fn set_send_buf_size(&mut self, size: usize) -> Result<(), Errno> {
         let mut inner = self.inner.protected.write().unwrap();
         match &mut inner.kind {
-            InodeSocketKind::PreSocket { send_buf_size, .. } => {
-                *send_buf_size = Some(size);
+            InodeSocketKind::PreSocket { props, .. }
+            | InodeSocketKind::RemoteSocket { props, .. } => {
+                props.send_buf_size = Some(size);
             }
             InodeSocketKind::TcpStream { socket, .. } => {
                 socket
@@ -744,8 +815,9 @@ impl InodeSocket {
     pub fn send_buf_size(&self) -> Result<usize, Errno> {
         let inner = self.inner.protected.read().unwrap();
         match &inner.kind {
-            InodeSocketKind::PreSocket { send_buf_size, .. } => {
-                Ok((*send_buf_size).unwrap_or_default())
+            InodeSocketKind::PreSocket { props, .. }
+            | InodeSocketKind::RemoteSocket { props, .. } => {
+                Ok(props.send_buf_size.unwrap_or_default())
             }
             InodeSocketKind::TcpStream { socket, .. } => {
                 socket.send_buf_size().map_err(net_error_into_wasi_err)
@@ -757,8 +829,9 @@ impl InodeSocket {
     pub fn set_recv_buf_size(&mut self, size: usize) -> Result<(), Errno> {
         let mut inner = self.inner.protected.write().unwrap();
         match &mut inner.kind {
-            InodeSocketKind::PreSocket { recv_buf_size, .. } => {
-                *recv_buf_size = Some(size);
+            InodeSocketKind::PreSocket { props, .. }
+            | InodeSocketKind::RemoteSocket { props, .. } => {
+                props.recv_buf_size = Some(size);
             }
             InodeSocketKind::TcpStream { socket, .. } => {
                 socket
@@ -773,8 +846,9 @@ impl InodeSocket {
     pub fn recv_buf_size(&self) -> Result<usize, Errno> {
         let inner = self.inner.protected.read().unwrap();
         match &inner.kind {
-            InodeSocketKind::PreSocket { recv_buf_size, .. } => {
-                Ok((*recv_buf_size).unwrap_or_default())
+            InodeSocketKind::PreSocket { props, .. }
+            | InodeSocketKind::RemoteSocket { props, .. } => {
+                Ok(props.recv_buf_size.unwrap_or_default())
             }
             InodeSocketKind::TcpStream { socket, .. } => {
                 socket.recv_buf_size().map_err(net_error_into_wasi_err)
@@ -789,6 +863,7 @@ impl InodeSocket {
             InodeSocketKind::TcpStream { socket, .. } => {
                 socket.set_linger(linger).map_err(net_error_into_wasi_err)
             }
+            InodeSocketKind::RemoteSocket { .. } => Ok(()),
             InodeSocketKind::PreSocket { .. } => Err(Errno::Io),
             _ => Err(Errno::Notsup),
         }
@@ -831,18 +906,13 @@ impl InodeSocket {
                 }
                 Ok(())
             }
-            InodeSocketKind::PreSocket {
-                read_timeout,
-                write_timeout,
-                connect_timeout,
-                accept_timeout,
-                ..
-            } => {
+            InodeSocketKind::PreSocket { props, .. }
+            | InodeSocketKind::RemoteSocket { props, .. } => {
                 match ty {
-                    TimeType::ConnectTimeout => *connect_timeout = timeout,
-                    TimeType::AcceptTimeout => *accept_timeout = timeout,
-                    TimeType::ReadTimeout => *read_timeout = timeout,
-                    TimeType::WriteTimeout => *write_timeout = timeout,
+                    TimeType::ConnectTimeout => props.connect_timeout = timeout,
+                    TimeType::AcceptTimeout => props.accept_timeout = timeout,
+                    TimeType::ReadTimeout => props.read_timeout = timeout,
+                    TimeType::WriteTimeout => props.write_timeout = timeout,
                     _ => return Err(Errno::Io),
                 }
                 Ok(())
@@ -867,17 +937,12 @@ impl InodeSocket {
                 TimeType::AcceptTimeout => *accept_timeout,
                 _ => return Err(Errno::Inval),
             }),
-            InodeSocketKind::PreSocket {
-                read_timeout,
-                write_timeout,
-                connect_timeout,
-                accept_timeout,
-                ..
-            } => match ty {
-                TimeType::ConnectTimeout => Ok(*connect_timeout),
-                TimeType::AcceptTimeout => Ok(*accept_timeout),
-                TimeType::ReadTimeout => Ok(*read_timeout),
-                TimeType::WriteTimeout => Ok(*write_timeout),
+            InodeSocketKind::PreSocket { props, .. }
+            | InodeSocketKind::RemoteSocket { props, .. } => match ty {
+                TimeType::ConnectTimeout => Ok(props.connect_timeout),
+                TimeType::AcceptTimeout => Ok(props.accept_timeout),
+                TimeType::ReadTimeout => Ok(props.read_timeout),
+                TimeType::WriteTimeout => Ok(props.write_timeout),
                 _ => Err(Errno::Inval),
             },
             _ => Err(Errno::Notsup),
@@ -893,6 +958,10 @@ impl InodeSocket {
             InodeSocketKind::UdpSocket { socket, .. } => {
                 socket.set_ttl(ttl).map_err(net_error_into_wasi_err)
             }
+            InodeSocketKind::RemoteSocket { ttl: set_ttl, .. } => {
+                *set_ttl = ttl;
+                Ok(())
+            }
             InodeSocketKind::PreSocket { .. } => Err(Errno::Io),
             _ => Err(Errno::Notsup),
         }
@@ -907,6 +976,7 @@ impl InodeSocket {
             InodeSocketKind::UdpSocket { socket, .. } => {
                 socket.ttl().map_err(net_error_into_wasi_err)
             }
+            InodeSocketKind::RemoteSocket { ttl, .. } => Ok(*ttl),
             InodeSocketKind::PreSocket { .. } => Err(Errno::Io),
             _ => Err(Errno::Notsup),
         }
@@ -918,6 +988,13 @@ impl InodeSocket {
             InodeSocketKind::UdpSocket { socket, .. } => socket
                 .set_multicast_ttl_v4(ttl)
                 .map_err(net_error_into_wasi_err),
+            InodeSocketKind::RemoteSocket {
+                multicast_ttl: set_ttl,
+                ..
+            } => {
+                *set_ttl = ttl;
+                Ok(())
+            }
             InodeSocketKind::PreSocket { .. } => Err(Errno::Io),
             _ => Err(Errno::Notsup),
         }
@@ -929,6 +1006,7 @@ impl InodeSocket {
             InodeSocketKind::UdpSocket { socket, .. } => {
                 socket.multicast_ttl_v4().map_err(net_error_into_wasi_err)
             }
+            InodeSocketKind::RemoteSocket { multicast_ttl, .. } => Ok(*multicast_ttl),
             InodeSocketKind::PreSocket { .. } => Err(Errno::Io),
             _ => Err(Errno::Notsup),
         }
@@ -940,6 +1018,7 @@ impl InodeSocket {
             InodeSocketKind::UdpSocket { socket, .. } => socket
                 .join_multicast_v4(multiaddr, iface)
                 .map_err(net_error_into_wasi_err),
+            InodeSocketKind::RemoteSocket { .. } => Ok(()),
             InodeSocketKind::PreSocket { .. } => Err(Errno::Io),
             _ => Err(Errno::Notsup),
         }
@@ -951,6 +1030,7 @@ impl InodeSocket {
             InodeSocketKind::UdpSocket { socket, .. } => socket
                 .leave_multicast_v4(multiaddr, iface)
                 .map_err(net_error_into_wasi_err),
+            InodeSocketKind::RemoteSocket { .. } => Ok(()),
             InodeSocketKind::PreSocket { .. } => Err(Errno::Io),
             _ => Err(Errno::Notsup),
         }
@@ -962,6 +1042,7 @@ impl InodeSocket {
             InodeSocketKind::UdpSocket { socket, .. } => socket
                 .join_multicast_v6(multiaddr, iface)
                 .map_err(net_error_into_wasi_err),
+            InodeSocketKind::RemoteSocket { .. } => Ok(()),
             InodeSocketKind::PreSocket { .. } => Err(Errno::Io),
             _ => Err(Errno::Notsup),
         }
@@ -973,6 +1054,7 @@ impl InodeSocket {
             InodeSocketKind::UdpSocket { socket, .. } => socket
                 .leave_multicast_v6(multiaddr, iface)
                 .map_err(net_error_into_wasi_err),
+            InodeSocketKind::RemoteSocket { .. } => Ok(()),
             InodeSocketKind::PreSocket { .. } => Err(Errno::Io),
             _ => Err(Errno::Notsup),
         }
@@ -1020,7 +1102,7 @@ impl InodeSocket {
                         InodeSocketKind::PreSocket { .. } => {
                             return Poll::Ready(Err(Errno::Notconn))
                         }
-                        InodeSocketKind::RemoteTcpStream { .. } => {
+                        InodeSocketKind::RemoteSocket { .. } => {
                             return Poll::Ready(Ok(self.data.len()))
                         }
                         _ => return Poll::Ready(Err(Errno::Notsup)),
@@ -1100,7 +1182,7 @@ impl InodeSocket {
                         InodeSocketKind::PreSocket { .. } => {
                             return Poll::Ready(Err(Errno::Notconn))
                         }
-                        InodeSocketKind::RemoteTcpStream { .. } => {
+                        InodeSocketKind::RemoteSocket { .. } => {
                             return Poll::Ready(Ok(self.data.len()))
                         }
                         _ => return Poll::Ready(Err(Errno::Notsup)),
@@ -1188,6 +1270,9 @@ impl InodeSocket {
                                 }
                             }
                         }
+                        InodeSocketKind::RemoteSocket { .. } => {
+                            return Poll::Pending;
+                        }
                         InodeSocketKind::PreSocket { .. } => {
                             return Poll::Ready(Err(Errno::Notconn))
                         }
@@ -1264,6 +1349,9 @@ impl InodeSocket {
                         InodeSocketKind::UdpSocket { socket, .. } => {
                             socket.try_recv_from(self.data)
                         }
+                        InodeSocketKind::RemoteSocket { .. } => {
+                            return Poll::Pending;
+                        }
                         InodeSocketKind::PreSocket { .. } => {
                             return Poll::Ready(Err(Errno::Notconn))
                         }
@@ -1310,6 +1398,7 @@ impl InodeSocket {
             InodeSocketKind::TcpStream { socket, .. } => {
                 socket.shutdown(how).map_err(net_error_into_wasi_err)?;
             }
+            InodeSocketKind::RemoteSocket { .. } => return Ok(()),
             InodeSocketKind::PreSocket { .. } => return Err(Errno::Notconn),
             _ => return Err(Errno::Notsup),
         }
@@ -1321,6 +1410,7 @@ impl InodeSocket {
             #[allow(clippy::match_like_matches_macro)]
             match &mut guard.kind {
                 InodeSocketKind::TcpStream { .. }
+                | InodeSocketKind::RemoteSocket { .. }
                 | InodeSocketKind::UdpSocket { .. }
                 | InodeSocketKind::Raw(..) => true,
                 _ => false,
@@ -1339,10 +1429,12 @@ impl InodeSocketProtected {
             InodeSocketKind::UdpSocket { socket, .. } => socket.remove_handler(),
             InodeSocketKind::Raw(socket) => socket.remove_handler(),
             InodeSocketKind::Icmp(socket) => socket.remove_handler(),
-            InodeSocketKind::PreSocket { handler, .. } => {
-                handler.take();
+            InodeSocketKind::PreSocket { props, .. } => {
+                props.handler.take();
+            }
+            InodeSocketKind::RemoteSocket { props, .. } => {
+                props.handler.take();
             }
-            InodeSocketKind::RemoteTcpStream { .. } => {}
         }
     }
 
@@ -1354,7 +1446,7 @@ impl InodeSocketProtected {
             InodeSocketKind::Raw(socket) => socket.poll_read_ready(cx),
             InodeSocketKind::Icmp(socket) => socket.poll_read_ready(cx),
             InodeSocketKind::PreSocket { .. } => Poll::Pending,
-            InodeSocketKind::RemoteTcpStream { .. } => Poll::Pending,
+            InodeSocketKind::RemoteSocket { .. } => Poll::Pending,
         }
         .map_err(net_error_into_io_err)
     }
@@ -1367,7 +1459,7 @@ impl InodeSocketProtected {
             InodeSocketKind::Raw(socket) => socket.poll_write_ready(cx),
             InodeSocketKind::Icmp(socket) => socket.poll_write_ready(cx),
             InodeSocketKind::PreSocket { .. } => Poll::Pending,
-            InodeSocketKind::RemoteTcpStream { .. } => Poll::Pending,
+            InodeSocketKind::RemoteSocket { .. } => Poll::Pending,
         }
         .map_err(net_error_into_io_err)
     }
@@ -1382,11 +1474,11 @@ impl InodeSocketProtected {
             InodeSocketKind::UdpSocket { socket, .. } => socket.set_handler(handler),
             InodeSocketKind::Raw(socket) => socket.set_handler(handler),
             InodeSocketKind::Icmp(socket) => socket.set_handler(handler),
-            InodeSocketKind::PreSocket { handler: h, .. } => {
-                h.replace(handler);
+            InodeSocketKind::PreSocket { props, .. }
+            | InodeSocketKind::RemoteSocket { props, .. } => {
+                props.handler.replace(handler);
                 Ok(())
             }
-            InodeSocketKind::RemoteTcpStream { .. } => Ok(()),
         }
     }
 }
