@@ -2,6 +2,8 @@ mod runner;
 
 #[cfg(feature = "webc_runner_rt_dcgi")]
 pub mod dcgi;
+#[cfg(feature = "webc_runner_rt_dproxy")]
+pub mod dproxy;
 #[cfg(feature = "webc_runner_rt_emscripten")]
 pub mod emscripten;
 pub mod wasi;
