@@ -13,9 +13,10 @@ use crate::{
     capabilities::Capabilities,
     journal::{DynJournal, SnapshotTrigger},
     runners::{wasi_common::CommonWasiOptions, MappedDirectory, MountedDirectory},
-    runtime::{module_cache::ModuleHash, task_manager::VirtualTaskManagerExt},
+    runtime::task_manager::VirtualTaskManagerExt,
     Runtime, WasiEnvBuilder, WasiError, WasiRuntimeError,
 };
+use wasmer_types::ModuleHash;
 
 use super::wasi_common::MappedCommand;
 
@@ -262,12 +263,38 @@ impl WasiRunner {
     ) -> Result<(), Error> {
         let wasi = webc::metadata::annotations::Wasi::new(program_name);
         let mut store = runtime.new_store();
-        let env = self.prepare_webc_env(program_name, &wasi, None, runtime, None)?;
+
+        let mut builder = self.prepare_webc_env(program_name, &wasi, None, runtime, None)?;
+
+        #[cfg(feature = "ctrlc")]
+        {
+            builder = builder.attach_ctrl_c();
+        }
+
+        #[cfg(feature = "journal")]
+        {
+            for trigger in self.wasi.snapshot_on.iter().cloned() {
+                builder.add_snapshot_trigger(trigger);
+            }
+            if self.wasi.snapshot_on.is_empty() && !self.wasi.journals.is_empty() {
+                for on in crate::journal::DEFAULT_SNAPSHOT_TRIGGERS {
+                    builder.add_snapshot_trigger(on);
+                }
+            }
+            if let Some(period) = self.wasi.snapshot_interval {
+                if self.wasi.journals.is_empty() {
+                    return Err(anyhow::format_err!(
+                            "If you specify a snapshot interval then you must also specify a journal file"
+                        ));
+                }
+                builder.with_snapshot_interval(period);
+            }
+        }
 
         if asyncify {
-            env.run_with_store_async(module.clone(), module_hash, store)?;
+            builder.run_with_store_async(module.clone(), module_hash, store)?;
         } else {
-            env.run_with_store_ext(module.clone(), module_hash, &mut store)?;
+            builder.run_with_store_ext(module.clone(), module_hash, &mut store)?;
         }
 
         Ok(())
