@@ -9,7 +9,8 @@ use anyhow::{Context, Error};
 use futures::{future::BoxFuture, StreamExt, TryStreamExt};
 use once_cell::sync::OnceCell;
 use petgraph::visit::EdgeRef;
-use virtual_fs::{FileSystem, OverlayFileSystem, WebcVolumeFileSystem};
+use virtual_fs::{FileSystem, OverlayFileSystem, UnionFileSystem, WebcVolumeFileSystem};
+use wasmer_config::package::PackageId;
 use webc::{
     compat::{Container, Volume},
     metadata::annotations::Atom as AtomAnnotation,
@@ -20,8 +21,8 @@ use crate::{
     runtime::{
         package_loader::PackageLoader,
         resolver::{
-            DependencyGraph, ItemLocation, PackageId, PackageSummary, Resolution,
-            ResolvedFileSystemMapping, ResolvedPackage,
+            DependencyGraph, ItemLocation, PackageSummary, Resolution, ResolvedFileSystemMapping,
+            ResolvedPackage,
         },
     },
 };
@@ -47,8 +48,7 @@ pub async fn load_package_tree(
     let file_system_memory_footprint = count_file_system(&fs, Path::new("/"));
 
     let loaded = BinaryPackage {
-        package_name: root.package_name.clone(),
-        version: root.version.clone(),
+        id: root.clone(),
         when_cached: crate::syscalls::platform_clock_time_get(
             wasmer_wasix_types::wasi::Snapshot0Clockid::Monotonic,
             1_000_000,
@@ -97,6 +97,7 @@ fn commands(
 
 /// Given a [`webc::metadata::Command`], figure out which atom it uses and load
 /// that atom into a [`BinaryPackageCommand`].
+#[tracing::instrument(skip_all, fields(%package_id, %command_name))]
 fn load_binary_command(
     package_id: &PackageId,
     command_name: &str,
@@ -122,7 +123,7 @@ fn load_binary_command(
 
     let package = &containers[package_id];
 
-    let webc = match dependency {
+    let (webc, resolved_package_id) = match dependency {
         Some(dep) => {
             let ix = resolution
                 .graph
@@ -137,24 +138,44 @@ fn load_binary_command(
                 .with_context(|| format!("Unable to find the \"{dep}\" dependency for the \"{command_name}\" command in \"{package_id}\""))?;
 
             let other_package = graph.node_weight(edge_reference.target()).unwrap();
-            &containers[&other_package.id]
+            let id = &other_package.id;
+
+            tracing::debug!(
+                dependency=%dep,
+                resolved_package_id=%id,
+                "command atom resolution: resolved dependency",
+            );
+            (&containers[id], id)
         }
-        None => package,
+        None => (package, package_id),
     };
 
     let atom = webc.get_atom(&atom_name);
 
     if atom.is_none() && cmd.annotations.is_empty() {
-        return Ok(legacy_atom_hack(webc, command_name, cmd));
+        tracing::info!("applying legacy atom hack");
+        return legacy_atom_hack(webc, command_name, cmd);
     }
 
+    let hash = webc.manifest().atom_signature(&atom_name)?.into();
+
     let atom = atom.with_context(|| {
+
+        let available_atoms = webc.atoms().keys().map(|x| x.as_str()).collect::<Vec<_>>().join(",");
+
+        tracing::warn!(
+            %atom_name,
+            %resolved_package_id,
+            %available_atoms,
+            "invalid command: could not find atom in package",
+        );
+
         format!(
-            "The '{command_name}' command uses the '{atom_name}' atom, but it isn't present in the WEBC file"
+            "The '{command_name}' command uses the '{atom_name}' atom, but it isn't present in the package: {resolved_package_id})"
         )
     })?;
 
-    let cmd = BinaryPackageCommand::new(command_name.to_string(), cmd.clone(), atom);
+    let cmd = BinaryPackageCommand::new(command_name.to_string(), cmd.clone(), atom, hash);
 
     Ok(Some(cmd))
 }
@@ -203,8 +224,12 @@ fn legacy_atom_hack(
     webc: &Container,
     command_name: &str,
     metadata: &webc::metadata::Command,
-) -> Option<BinaryPackageCommand> {
-    let (name, atom) = webc.atoms().into_iter().next()?;
+) -> Result<Option<BinaryPackageCommand>, anyhow::Error> {
+    let (name, atom) = webc
+        .atoms()
+        .into_iter()
+        .next()
+        .ok_or_else(|| anyhow::Error::msg("container does not have any atom"))?;
 
     tracing::debug!(
         command_name,
@@ -213,11 +238,14 @@ fn legacy_atom_hack(
         "(hack) The command metadata is malformed. Falling back to the first atom in the WEBC file",
     );
 
-    Some(BinaryPackageCommand::new(
+    let hash = webc.manifest().atom_signature(&name)?.into();
+
+    Ok(Some(BinaryPackageCommand::new(
         command_name.to_string(),
         metadata.clone(),
         atom,
-    ))
+        hash,
+    )))
 }
 
 async fn fetch_dependencies(
@@ -285,32 +313,121 @@ fn count_file_system(fs: &dyn FileSystem, path: &Path) -> u64 {
 
 /// Given a set of [`ResolvedFileSystemMapping`]s and the [`Container`] for each
 /// package in a dependency tree, construct the resulting filesystem.
-///
-/// # Note to future readers
-///
-/// Sooo... this code is a bit convoluted because we're constrained by the
-/// filesystem implementations we've got available.
-///
-/// Ideally, we would create a WebcVolumeFileSystem for each volume we're
-/// using, then we'd have a single "union" filesystem which lets you mount
-/// filesystem objects under various paths and can deal with conflicts.
-///
-/// The OverlayFileSystem lets us make files from multiple filesystem
-/// implementations available at the same time, however all of the
-/// filesystems will be mounted at "/", when the user wants to mount volumes
-/// at arbitrary locations.
-///
-/// The TmpFileSystem *does* allow mounting at non-root paths, however it can't
-/// handle nested paths (e.g. mounting to "/lib" and "/lib/python3.10" - see
-/// <https://github.com/wasmerio/wasmer/issues/3678> for more) and you aren't
-/// allowed to mount to "/" because it's a special directory that already
-/// exists.
-///
-/// As a result, we'll duct-tape things together and hope for the best 🤞
 fn filesystem(
     packages: &HashMap<PackageId, Container>,
     pkg: &ResolvedPackage,
-) -> Result<impl FileSystem + Send + Sync, Error> {
+) -> Result<Box<dyn FileSystem + Send + Sync>, Error> {
+    if pkg.filesystem.is_empty() {
+        return Ok(Box::new(OverlayFileSystem::<
+            virtual_fs::EmptyFileSystem,
+            Vec<WebcVolumeFileSystem>,
+        >::new(
+            virtual_fs::EmptyFileSystem::default(), vec![]
+        )));
+    }
+
+    let mut found_v2 = false;
+    let mut found_v3 = false;
+
+    for ResolvedFileSystemMapping { package, .. } in &pkg.filesystem {
+        let container = packages.get(package).with_context(|| {
+            format!(
+                "\"{}\" wants to use the \"{}\" package, but it isn't in the dependency tree",
+                pkg.root_package, package,
+            )
+        })?;
+
+        found_v2 |= container.version() == webc::Version::V2;
+        found_v3 |= container.version() == webc::Version::V3;
+    }
+
+    if found_v3 && !found_v2 {
+        filesystem_v3(packages, pkg)
+    } else {
+        filesystem_v2(packages, pkg)
+    }
+}
+
+/// Build the filesystem for webc v3 packages.
+fn filesystem_v3(
+    packages: &HashMap<PackageId, Container>,
+    pkg: &ResolvedPackage,
+) -> Result<Box<dyn FileSystem + Send + Sync>, Error> {
+    let mut volumes: HashMap<&PackageId, BTreeMap<String, Volume>> = HashMap::new();
+
+    let mut mountings: Vec<_> = pkg.filesystem.iter().collect();
+    mountings.sort_by_key(|m| std::cmp::Reverse(m.mount_path.as_path()));
+
+    let mut union_fs = UnionFileSystem::new();
+
+    for ResolvedFileSystemMapping {
+        mount_path,
+        volume_name,
+        package,
+        ..
+    } in &pkg.filesystem
+    {
+        // Note: We want to reuse existing Volume instances if we can. That way
+        // we can keep the memory usage down. A webc::compat::Volume is
+        // reference-counted, anyway.
+        // looks like we need to insert it
+        let container = packages.get(package).with_context(|| {
+            format!(
+                "\"{}\" wants to use the \"{}\" package, but it isn't in the dependency tree",
+                pkg.root_package, package,
+            )
+        })?;
+        let container_volumes = match volumes.entry(package) {
+            std::collections::hash_map::Entry::Occupied(entry) => &*entry.into_mut(),
+            std::collections::hash_map::Entry::Vacant(entry) => &*entry.insert(container.volumes()),
+        };
+
+        let volume = container_volumes.get(volume_name).with_context(|| {
+            format!("The \"{package}\" package doesn't have a \"{volume_name}\" volume")
+        })?;
+
+        let webc_vol = WebcVolumeFileSystem::new(volume.clone());
+        union_fs.mount(
+            volume_name,
+            mount_path.to_str().unwrap(),
+            false,
+            Box::new(webc_vol),
+            None,
+        );
+    }
+
+    let fs = OverlayFileSystem::new(virtual_fs::EmptyFileSystem::default(), [union_fs]);
+
+    Ok(Box::new(fs))
+}
+
+/// Build the filesystem for webc v2 packages.
+///
+// # Note to future readers
+//
+// Sooo... this code is a bit convoluted because we're constrained by the
+// filesystem implementations we've got available.
+//
+// Ideally, we would create a WebcVolumeFileSystem for each volume we're
+// using, then we'd have a single "union" filesystem which lets you mount
+// filesystem objects under various paths and can deal with conflicts.
+//
+// The OverlayFileSystem lets us make files from multiple filesystem
+// implementations available at the same time, however all of the
+// filesystems will be mounted at "/", when the user wants to mount volumes
+// at arbitrary locations.
+//
+// The TmpFileSystem *does* allow mounting at non-root paths, however it can't
+// handle nested paths (e.g. mounting to "/lib" and "/lib/python3.10" - see
+// <https://github.com/wasmerio/wasmer/issues/3678> for more) and you aren't
+// allowed to mount to "/" because it's a special directory that already
+// exists.
+//
+// As a result, we'll duct-tape things together and hope for the best 🤞
+fn filesystem_v2(
+    packages: &HashMap<PackageId, Container>,
+    pkg: &ResolvedPackage,
+) -> Result<Box<dyn FileSystem + Send + Sync>, Error> {
     let mut filesystems = Vec::new();
     let mut volumes: HashMap<&PackageId, BTreeMap<String, Volume>> = HashMap::new();
 
@@ -345,27 +462,43 @@ fn filesystem(
             format!("The \"{package}\" package doesn't have a \"{volume_name}\" volume")
         })?;
 
-        let original_path = PathBuf::from(original_path);
         let mount_path = mount_path.clone();
         // Get a filesystem which will map "$mount_dir/some-path" to
         // "$original_path/some-path" on the original volume
-        let fs =
-            MappedPathFileSystem::new(WebcVolumeFileSystem::new(volume.clone()), move |path| {
-                let without_mount_dir = path
-                    .strip_prefix(&mount_path)
-                    .map_err(|_| virtual_fs::FsError::BaseNotDirectory)?;
-                let path_on_original_volume = original_path.join(without_mount_dir);
-                Ok(path_on_original_volume)
-            });
+        let fs = if let Some(original) = original_path {
+            let original = PathBuf::from(original);
+
+            MappedPathFileSystem::new(
+                WebcVolumeFileSystem::new(volume.clone()),
+                Box::new(move |path: &Path| {
+                    let without_mount_dir = path
+                        .strip_prefix(&mount_path)
+                        .map_err(|_| virtual_fs::FsError::BaseNotDirectory)?;
+                    Ok(original.join(without_mount_dir))
+                }) as DynPathMapper,
+            )
+        } else {
+            MappedPathFileSystem::new(
+                WebcVolumeFileSystem::new(volume.clone()),
+                Box::new(move |path: &Path| {
+                    let without_mount_dir = path
+                        .strip_prefix(&mount_path)
+                        .map_err(|_| virtual_fs::FsError::BaseNotDirectory)?;
+                    Ok(without_mount_dir.to_owned())
+                }) as DynPathMapper,
+            )
+        };
 
         filesystems.push(fs);
     }
 
     let fs = OverlayFileSystem::new(virtual_fs::EmptyFileSystem::default(), filesystems);
 
-    Ok(fs)
+    Ok(Box::new(fs))
 }
 
+type DynPathMapper = Box<dyn Fn(&Path) -> Result<PathBuf, virtual_fs::FsError> + Send + Sync>;
+
 /// A [`FileSystem`] implementation that lets you map the [`Path`] to something
 /// else.
 #[derive(Clone, PartialEq)]
